"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const command_1 = require("@oclif/command/lib/command");
const dirTree = require("directory-tree");
const fs = require("fs");
const path = require("path");
const copy = require("recursive-copy");
const rimraf = require("rimraf");
const through = require("through2");
const conf_1 = require("../services/conf");
const logger_1 = require("../services/logger");
const markdown_1 = require("../services/markdown");
const render_1 = require("../services/render");
const { resolve } = path;
class DocsBase extends command_1.default {
    get navs() {
        const navs = [
            { title: '文档', main: true },
            { title: 'FAQ', url: '/FAQ.html' },
            { title: '日志', url: '/rde/changelog.html' },
        ];
        if (this.isRdt) {
            const nav = { title: '速查', url: '/rde/cheat.sheet.html' };
            navs.splice(1, 0, nav);
            return navs;
        }
        return navs;
    }
    get userScripts() {
        const { docs } = this.localConfig;
        return docs.userScripts || '';
    }
    get frameworkScripts() {
        const { framework = 'vue' } = this.localConfig;
        if (!conf_1.default.frameworks[framework]) {
            throw Error(`Rde doesn't support framework: ${framework}`);
        }
        const { cdn = [] } = conf_1.default.frameworks[framework];
        let scripts = '';
        cdn.forEach(url => {
            scripts += `<script src="${url}"><script>`;
        });
        return scripts;
    }
    get option() {
        return {
            overwrite: true,
            rename(filePath) {
                if (filePath.endsWith('.md')) {
                    const dirname = path.dirname(filePath);
                    const filename = path.basename(filePath, '.md');
                    return `${dirname}/${filename}.html`;
                }
                return filePath;
            },
            filter(src) {
                src = `${conf_1.default.docsDir}/${src}`;
                if (DocsBase.excludeDirRule.test(src)) {
                    return false;
                }
                return true;
            },
            transform: () => {
                return through((chunk, _enc, done) => {
                    const content = markdown_1.default.render(chunk.toString());
                    const { loadTemplate: load } = render_1.default;
                    const index = load('docs/index');
                    const style = load('docs/style');
                    const layout = load('docs/layout');
                    const output = render_1.default.render(index, {
                        title: 'RDE Suite',
                        content,
                        navs: JSON.stringify(this.navs),
                        pages: JSON.stringify(this.pages),
                    }, ['<%', '%>'], {
                        style,
                        layout,
                        userScripts: this.userScripts,
                        frameworkScripts: this.frameworkScripts,
                    });
                    done(null, output);
                });
            }
        };
    }
    async init() {
        const { name } = require(resolve(conf_1.default.cwd, './package.json'));
        if (!name.endsWith('-rdt') && !name.endsWith('-rds')) {
            throw Error('wrong package name format, please end it with -rdt or -rds');
        }
        else {
            this.isRdt = name.endsWith('-rdt');
        }
        const homepagePath = resolve(conf_1.default.docsDir, 'index.md');
        const faqPath = resolve(conf_1.default.docsDir, 'faq.md');
        if (!fs.existsSync(homepagePath) || !fs.existsSync(faqPath)) {
            throw Error('cannot find index.md or faq.md in your _docs dir, please provide');
        }
        if (!fs.existsSync(conf_1.default.docsDir)) {
            throw Error('cannot find _docs dir, please provide');
        }
        if (this.isRdt) {
            this.localConfig = require(`${conf_1.default.cwd}/${conf_1.default.rdcConfName}`);
        }
        else {
            this.localConfig = require(`${conf_1.default.cwd}/${conf_1.default.rdsConfName}`);
        }
        this.pages = this.getPages();
        await this.render();
        process.on('SIGINT', () => {
            rimraf.sync(conf_1.default.docsPagesDir);
            process.exit();
        });
    }
    async postRun() { }
    async catch(e) {
        logger_1.logger.error(e.message);
        this.exit(1);
    }
    async finally(e) {
        if (!e) {
            await this.postRun();
        }
    }
    getPages() {
        const pages = [];
        const files = dirTree(conf_1.default.docsDir, {
            extensions: /\.md$/,
            exclude: DocsBase.excludeDirRule,
        });
        files.children.forEach(file => {
            if (file.type === 'file') {
                let content = fs.readFileSync(file.path).toString();
                markdown_1.default.render(content);
                const { meta = {} } = markdown_1.default;
                pages.push({
                    title: meta.title || file.name,
                    url: `/${file.name.slice(0, -3)}.html`
                });
            }
            if (file.type === 'directory') {
                let category;
                const page = {
                    title: file.name,
                    children: file.children.map(child => {
                        let content = fs.readFileSync(child.path).toString();
                        markdown_1.default.render(content);
                        const { meta = {} } = markdown_1.default;
                        if (meta.category) {
                            category = meta.category;
                        }
                        return {
                            title: meta.title || child.name,
                            url: `/${file.name}/${child.name.slice(0, -3)}.html`
                        };
                    })
                };
                page.title = category || file.name;
                pages.push(page);
            }
        });
        return pages;
    }
    async render() {
        const { docsDir, docsPagesDir } = conf_1.default;
        await copy(docsDir, docsPagesDir, this.option);
    }
}
DocsBase.excludeDirRule = /^_docs\/[^\/]+\/(?!([^\/]*\.md$))+/;
exports.default = DocsBase;
